﻿using System.Data.Entity;
using System.Security.Claims;
using System.Threading.Tasks;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using System.Data.Entity.ModelConfiguration;
using System.Collections.Generic;
using System;
using System.ComponentModel.DataAnnotations;

namespace WebAuth.Models
{
    /// <summary>
    /// ApplicationUser
    /// </summary>
    public class ApplicationUser : IdentityUser
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ApplicationUser"/> class.
        /// </summary>
        public ApplicationUser() : base() { this.Departments = new List<UserDepartment>(); }
        /// <summary>
        /// Initializes a new instance of the <see cref="ApplicationUser"/> class.
        /// </summary>
        /// <param name="userName"></param>
        public ApplicationUser(string userName) : base(userName) { }

        /// <summary>
        /// 所属部门
        /// </summary>
        public virtual ICollection<UserDepartment> Departments { get; set; }


        /// <summary>
        /// Gets or sets the name of the chinese.
        /// </summary>
        /// <value>
        /// The name of the chinese.
        /// </value>
        [StringLength(30, ErrorMessage = "The {0} must be at least {2} characters long.", MinimumLength = 2)]
        public string ChineseName { get; set; }

        public string HeaderPhoto { get; set; }


        /// <summary>
        /// Generates the user identity asynchronous.
        /// </summary>
        /// <param name="manager">The manager.</param>
        /// <returns></returns>
        public async Task<ClaimsIdentity> GenerateUserIdentityAsync(UserManager<ApplicationUser> manager)
        {
            // Note the authenticationType must match the one defined in CookieAuthenticationOptions.AuthenticationType
            var userIdentity = await manager.CreateIdentityAsync(this, DefaultAuthenticationTypes.ApplicationCookie);
            // Add custom user claims here
            return userIdentity;
        }

    }



  
}